/* step4 エアーホッケー風ゲーム 2017-06-30 　*/
#include "dm4.h"

#define FENCE_NUM 6
static int STEPS = 0;   // シミュレーションのステップ数
double red[3]    = {1.3, 0.0, 0.0}; // 赤色
double yellow[3] = {1.3, 1.3, 0.0}; // 黄色
double green[3]  = {0.0, 0.8, 0.0}; // 緑色

dmObject ball, field, fence[FENCE_NUM], bar, bar_ai; // ドミノ, フィールド, 柵
dmFunctions dmf; // 描画関数の構造体

/***  シミュレーションループ　***/
void simLoop(int pause) {
    int i;

    if (STEPS == 1) {
        dmAddForce(&ball, 0.8, 0, 0); // ボールの落下速度を決める
    }
    dmWorldQuickStep(); // シミュレーションを１ステップ進める(高速版)

    dmDraw(&ball);
    dmDraw(&bar);
    dmDraw(&bar_ai);
    dmDraw(&field);
    for (i =0; i < FENCE_NUM; i++) {
        dmDraw(&fence[i]);
    }

    STEPS++;
}

void resetSim(int n) {
    double m = 0.1; // 質量
    double side[3] = {0.2, 0.05, 0.5}; // サイズ
    double R[12] = {1,0,0,0, 0,1,0,0, 0,0,1,0}; // 姿勢
    double R_BAR[12] = {1,0,0,0, 0,1,0,0, 0,0,1,0}; // 姿勢
    double R_BAR_AI[12] = {1,0,0,0, 0,1,0,0, 0,0,1,0}; // 姿勢
    // double ball_pos[3]={-0.90, 0.25, 0.7}, ball_r = 0.03, ball_m = 0.1; // ボールの位置、半径、質量
    double ball_pos[3]= {-0.5, 0, 0.525}, ball_r = 0.03, ball_m = 0.1; // ボールの位置、半径、質量
    double bar_pos[3]= {-0.08, 0, 0.5}, bar_side[3] = {0.01, 0.1, 0.1}, bar_m = 0.1; // ボールの位置、半径、質量
    double bar_ai_pos[3]= {-0.9, 0, 0.6};
    double field_pos[3]= {-0.5, 0, 0.5}, field_side[3] = {1, 1, 0.01}, field_m; // スロープの位置、サイズ, 質量
    double fence_pos[FENCE_NUM][3] = {{-1.0, 0.3125, 0.6}, {-1.0,-0.3125, 0.6},{0.0, 0.3125, 0.6},{0.0,-0.3125, 0.6},{-0.5, -0.5, 0.6},{-0.5, 0.5, 0.6}};
    double fence_side[FENCE_NUM][3] = {{0.05, 0.425, 0.1}, {0.05, 0.425, 0.1},{0.05, 0.425, 0.1},
        {0.05, 0.425, 0.1},{1.0, 0.05, 0.1}, {1.0, 0.05, 0.1}
    };
    double fence_m[3] = {0.1, 0.1, 0.1};
    double slope_R[12]= {1,0,0,0, 0,1,0,0, 0,0,0};
    int i;

    STEPS = 0;

    // シミュレーションの終了
    if (STEPS != 0) {
        dmClose();
    }

    dmInit(); // 初期化

    //  ボールの生成
    dmCreateSphere(&ball, ball_pos, R, ball_m, ball_r, red);

    // バーの生成
    dRFromAxisAndAngle(R_BAR,0,0,1,M_PI/6);
    dRFromAxisAndAngle(R_BAR_AI,0,0,1,0);
    dmCreateBoxStatic(&bar, bar_pos, R_BAR, bar_m, bar_side, yellow);
    dmCreateBoxStatic(&bar_ai, bar_ai_pos, R_BAR_AI, bar_m, bar_side, red);

    // フィールドの生成
    dmCreateBoxStatic(&field, field_pos, R, field_m, field_side, green);
    for (i =0; i < FENCE_NUM; i++) {
        dmCreateBoxStatic(&fence[i], fence_pos[i], R, fence_m[i], fence_side[i], green);
    }
}

/*** キ―入力関数 ***/
void command(int cmd) {
    switch (cmd) {
    case 'r':
    case 'R':
        resetSim(0);
        break;
    default:
        printf("Input r, R, f, F key \n");
        break;
    }
}

/*** カメラの位置と姿勢設定 ***/
void setCamera() {
    float x =  -0.5, y = 0.0, z = 1.5;    // カメラの位置
    float roll = 0, pitch = -90, yaw = -180; // カメラの方向[°]
    dmSetCamera(x,y,z,roll,pitch,yaw);  // カメラの設定
}

/*** 描画用構造体の設定　***/
void setDraw() {
    dmf.start       = &setCamera;
    dmf.step        = &simLoop;
    dmf.command = &command;
}

/*** main関数 ***/
int main() {
    resetSim(0); // シミュレーションのリセット
    setDraw();   // 描画関数の設定

    dmLoop(800, 600, &dmf);  // ウインドウの幅，高, ループ関数，コマンド関数
    dmClose(); // 終了

    return 0;
}




